/*
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright 2006-2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.kordamp.json.processors;

import java.util.Iterator;
import java.util.Set;

/**
 * @author Andres Almiray
 */
public class StartsWithJsonBeanProcessorMatcher extends JsonBeanProcessorMatcher {
    private String pattern;

    public StartsWithJsonBeanProcessorMatcher(String pattern) {
        this.pattern = pattern;
    }

    public Object getMatch(Class target, Set set) {
        if (target != null && set != null && target.getName()
            .startsWith(pattern)) {
            for (Iterator i = set.iterator(); i.hasNext(); ) {
                Class c = (Class) i.next();
                if (c.getName()
                    .startsWith(pattern)) {
                    return c;
                }
            }
        }

        return null;
    }
}